package org.fuseri.moduleexercise.common;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;

import java.util.Optional;

/**
 * Dummy interface of repository. Later will be replaced by JpaRepository.
 *
 * @param <T>  entity
 * @param <ID> entity ID
 */
public interface DomainRepository<T, ID> {

    /**
     * Save the specified entity
     *
     * @param entity entity to be saved
     * @return created entity
     */
    T save(T entity);

    /**
     * Find entity by ID
     *
     * @param id ID of entity to be found
     * @return {@code Optional} containing the found entity,
     * or an empty {@code Optional} if no such entity exists
     */
    Optional<T> findById(ID id);

    /**
     * Retrieve a page of entities according to the specified pagination information
     *
     * @param pageRequest the pagination information for the query
     * @return a page of entities that satisfy the pagination criteria
     */
    Page<T> findAll(PageRequest pageRequest);

    /**
     * Update entity
     *
     * @param entity entity to update
     * @return updated entity
     */
    T update(T entity);

    /**
     * Delete the entity with the specified id
     * Note that this does not do cascade deleting.
     * We will have cascade deleting with usage of JpaRepository
     *
     * @param id the id of the entity to be deleted
     */
    void deleteById(ID id);
}