package org.fuseri.moduleexercise.question;

import org.fuseri.moduleexercise.common.DomainRepositoryImpl;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * An implementation of the QuestionRepository interface
 * Provides access to Question entities stored in a data source
 */
@Repository
public class QuestionRepositoryImpl extends DomainRepositoryImpl<Question> implements QuestionRepository {

    /**
     * Find a page of questions associated with the exercise with the specified ID
     *
     * @param exerciseId  the ID of the exercise to find questions for
     * @param pageRequest the page request specifying the page number and page size
     * @return a page of questions associated with the specified exercise
     */
    @Override
    public Page<Question> findByExerciseId(String exerciseId, PageRequest pageRequest) {
        List<Question> filteredQuestions = getItems()
                .stream()
                .filter(e -> e.getExerciseId().equals(exerciseId))
                .skip(pageRequest.getOffset())
                .limit(pageRequest.getPageSize())
                .toList();

        return new PageImpl<>(filteredQuestions, pageRequest, filteredQuestions.size());
    }
}
